'use strict';
angular.module('aom.directives')
  .directive('aomMediaList', function ($timeout, $ionicScrollDelegate, aomAttachmentsSrv, aomMediaSrv, aomJobSrv, aomResourcesSrv, aomBackSrv, aomPopupSrv, aomLogSrv) {
    return {
      scope: {

      },
      controller: function ($scope) {
        $scope.data = {
          attachmentSrvData: aomAttachmentsSrv.data,
          resources: aomResourcesSrv.data.resources
        };

        function goBack() {
          if (!aomMediaSrv.data.isSelectingMedia) {
            aomBackSrv.exit(true);
          }
        }

        $scope.$on('AttachmentOptions', function () {
          showAttachmentOptions();
        });

        function showAttachmentOptions() {
          var type = aomAttachmentsSrv.data.options.Type;
          var shouldShowAddDocumentOption = !aomAttachmentsSrv.data.options.isField;
          var params = {
            source: {
              photo: (type === 'All' || type === 'Photo'),
              video: (type === 'All' || type === 'Video'),
              gallery: true,
              sharedFiles: shouldShowAddDocumentOption
            }
          };
          aomMediaSrv.showMediaOptions(params, $scope)
            .then(function (media) {
              return aomAttachmentsSrv.addAttachment(aomJobSrv.data.selectedJob.Id, media.file, null, media.source);
            })
            .then(function (name) {
              $timeout(function () {
                $ionicScrollDelegate.$getByHandle('media-list').scrollBottom(true);
              }, 100);
              aomAttachmentsSrv.editAttachment(name, $scope);
            })
            .catch(function (err) {
              if (err !== 'UserCancelled') {
                aomPopupSrv.alert('AddAttachmentFailed', {
                  title: aomResourcesSrv.data.resources.Common.Error,
                  template: aomResourcesSrv.data.resources.Errors.AttachmentFail,
                  onDismiss: function () { }
                });
                aomLogSrv.log({ message: 'Failed to save/edit media attachment.', error: err }, aomLogSrv.levels.user, aomLogSrv.severities.warn);
              }
            })
            .finally(function () {
              aomMediaSrv.data.isSelectingMedia = false;
            });
        }

        if (aomAttachmentsSrv.data.attachments.length === 0 && !aomAttachmentsSrv.data.options.readonly) {
          showAttachmentOptions();
        }

        $scope.$on('$ionicView.loaded', function () {
          aomBackSrv.init(goBack);
        });
      },
      templateUrl: 'App/Components/MediaList/mediaList.html'
    };
  });