'use strict';
angular.module('aom.services')
  .factory('aomAuthInterceptorSrv', function ($q, $timeout, $injector, aomLogSrv, constants) {
    var authInterceptorService = {};

    var _request = function (config) {
      var target = config.url.split('/')[2];
      if (target !== 'slack.com' && target !== 's3.eu-west-2.amazonaws.com') {
        config.headers = config.headers || {};
        var authService = $injector.get('aomAuthSrv');
        config.headers.Authorization = 'Bearer ' + authService.data.authData.token;
      }
      return config;
    };

    var _responseError = function (rejection) {
      var authService = $injector.get('aomAuthSrv');
      var resources = $injector.get('aomResourcesSrv');
      var state = $injector.get('$state');
      var popup = $injector.get('$ionicPopup');
      // Zero check below is due to server not attaching cors headers 
      if (rejection.status === 401 && !authService.data.isReauthenticating) {
        // Try and logon again
        authService.data.isReauthenticating = true;
        authService.logon(resources.data.lastUser.logonRequest)
          .then(() => {
            authService.data.isReauthenticating = false;
          })
          .catch(function (response) {
            $timeout(() => {
              authService.data.isReauthenticating = true;
            });
            if (!state.is('logon')) {
              popup.alert({
                template: resources.data.resources.Errors.AuthenticationFailureMessage,
                buttons: [
                  {
                    text: resources.data.resources.Common.Okay,
                    type: 'button-positive',
                    onTap: function () {
                      authService.data.isReauthenticating = false;
                      state.go('logon');
                    }
                  }
                ]
              });
              aomLogSrv.log({ message: 'Failed to Authenticate For Second Time.', error: response }, aomLogSrv.levels.high, aomLogSrv.severities.err);
            }
          });
      }
      return $q.reject(rejection);
    };

    authInterceptorService.request = _request;
    authInterceptorService.responseError = _responseError;

    return authInterceptorService;
  });