'use strict';
angular.module('aom.services')
  .factory('aomAutoSaveSrv', function ($q, $timeout, aomDatabaseSrv, aomPopupSrv, aomResourcesSrv, aomLogSrv) {
    var autoSave = {},
      saveTempJob = null;

    autoSave.data = {
      selectedJob: {}
    };

    function tempJobSaved() {
      var query = {
        Recovery: { detail_json: null },
        where: {
          Id: autoSave.data.selectedJob.Id
        }
      };
      return aomDatabaseSrv.selectQuery(query)
        .then(function (job) {
          return job.rows;
        })
        .catch(function () {
          return $q.when([]);
        });
    }

    function deleteTempJob(tempJobId) {
      return aomDatabaseSrv.deleteQuery({
        Recovery: {
          id: tempJobId
        }
      });
    }

    autoSave.handleRecovered = function () {
      var def = $q.defer();
      tempJobSaved()
        .then(function (jobs) {
          if (jobs.length === 1 && jobs[0].Version === autoSave.data.selectedJob.Version) {
            autoSave.data.selectedJob = angular.copy(jobs[0], autoSave.data.selectedJob);
            def.resolve();
          } else {
            def.resolve();
          }
        })
        .catch(function (err) {
          def.reject(err);
        });
      return def.promise;
    };

    autoSave.saveTempJob = function () {
      if (saveTempJob !== null) {
        $timeout.cancel(saveTempJob);
      }
      saveTempJob = $timeout(function () {
        var query = {
          Recovery: {
            id: autoSave.data.selectedJob.Id,
            detail_json: angular.copy(autoSave.data.selectedJob)
          }
        };
        aomDatabaseSrv.insertQuery(query, true, false);
      }, 2000);
    };

    autoSave.cancelSaveTempJob = function () {
      if (saveTempJob !== null) {
        $timeout.cancel(saveTempJob);
      }
    };

    autoSave.rollbackAttachmentUpdates = function rollbackAttachmentUpdates(jobId) {
      return aomDatabaseSrv.deleteQuery({ Attachments: { jobId: jobId, saveState: 'add' } })
        .then(function () {
          return aomDatabaseSrv.updateQuery({ Attachments: { saveState: null }, where: { jobId: jobId, saveState: 'delete' } });
        });
    };

    autoSave.completeSaveJob = function (originalId, wasTemplate) {
      return deleteTempJob(originalId)
        .finally(function () {
          if (wasTemplate) {
            return aomDatabaseSrv.selectQuery(
              {
                Templates: {
                  detail_json: null
                },
                where: {
                  Id: originalId
                }
              })
              .then(function (originalTemplate) {
                angular.copy(originalTemplate.rows[0], autoSave.data.selectedJob);
                return $q.when();
              })
              .catch(function (err) {
                aomLogSrv.log({ message: 'Error in deleting temp job', error: err }, aomLogSrv.levels.high, aomLogSrv.severities.err);
              });
          } else {
            return $q.when();
          }
        });
    };

    autoSave.discardJob = function () {
      return deleteTempJob(autoSave.data.selectedJob.Id)
        .then(function () {
          var query = {
            where: {
              Id: autoSave.data.selectedJob.Id
            }
          };
          if (autoSave.data.selectedJob.isTemplate) {
            query.Templates = { detail_json: null };
          } else {
            query.Jobs = { detail_json: null };
          }
          return aomDatabaseSrv.selectQuery(query);
        })
        .then(function (job) {
          angular.copy(job.rows[0], autoSave.data.selectedJob);
        })
        .catch(function (err) {
          aomLogSrv.log({ message: 'Error in discarding the temp job', error: err }, aomLogSrv.levels.high, aomLogSrv.severities.err);
        });
    };

    return autoSave;
  });