
'use strict';
angular.module('aom.services')
  .factory('aomMarkersSrv', function ($rootScope, aomResourcesSrv, aomGoogleMapsSrv, aomComponentsSrv, aomPopupSrv) {
    var markers = {};

    markers.data = {
      selectedJob: null
    };

    function showPrompt(scope) {

      var aButtons = [{
        text: aomResourcesSrv.data.resources.Common.Done,
        type: 'button-positive'
      }];

      if ((scope.marker.isWorkLocation && !scope.workLocationFromServer) || !scope.marker.isWorkLocation) {
        aButtons.push({
          text: aomResourcesSrv.data.resources.Common.Remove,
          type: 'button-assertive button-outline',
          tapMethod: function () {
            if (scope.marker.isWorkLocation) {
              scope.jsProperties.workLocationSet = false;
              markers.data.selectedJob.WorkLocation.Address = null;
              markers.data.selectedJob.WorkLocation.Latitude = null;
              markers.data.selectedJob.WorkLocation.Longitude = null;
            }
            delete scope.markersOnJob[scope.marker.id];
          }
        });
      }

      aomPopupSrv.wrapper({
        id: 'marker',
        templateUrl: 'App/Common/Templates/markerInfoTemplate.html',
        title: aomResourcesSrv.data.resources.AomActionSheetOptions.EditComments,
        scope: scope,
        buttons: aButtons
      });
    }

    markers.regenerateMarkers = function (job) {
      var oldMarkers = angular.copy(job.markersOnJob);
      job.markersOnJob = {};
      job.markerBounds = [];
      markers.data.selectedJob = job;
      for (var prop in oldMarkers) {
        if (oldMarkers.hasOwnProperty(prop)) {
          markers.addMarker(job, oldMarkers[prop]);
        }
      }
    };

    markers.addMarker = function (job, aMarker) {
      var mapOptions = aomResourcesSrv.data.resources.MapOptions;
      var markerIcon = {
        type: 'awesomeMarker',
        markerColor: aMarker.isWorkLocation ? mapOptions.MarkerColorWorkLocation : mapOptions.MarkerColorCustomLocation
      };
      var id = aomComponentsSrv.generateUniqueId();
      var latitude = parseFloat(aMarker.lat);
      var longitude = parseFloat(aMarker.lng);

      job.markersOnJob[id] = {
        id: id,
        lat: latitude,
        lng: longitude,
        icon: markerIcon,
        draggable: true,
        zoomAnimation: true,
        markerZoomAnimation: true,
        message: '<a ng-click="edit()">{{marker.comment}}<br/>{{marker.address}}</a>',
        comment: aMarker.comment ? aMarker.comment : mapOptions.DefaultMarkerComment,
        address: aMarker.address ? aMarker.address : '',
        locationChanged: aMarker.locationChanged === undefined ? true : aMarker.locationChanged,
        dateTime: aMarker.dateTime ? aMarker.dateTime : moment().format(),
        getMessageScope: function () {
          var scope = $rootScope.$new();
          scope.marker = job.markersOnJob[this.id];
          scope.markersOnJob = job.markersOnJob;
          scope.workLocationFromServer = job.workLocationFromServer;
          scope.jsProperties = job.jsProperties;
          scope.markerId = this.id;

          if (scope.marker.locationChanged) {
            scope.marker.address = '';
            aomGoogleMapsSrv.getAddress(scope.marker.lat, scope.marker.lng)
              .then(function (address) {
                scope.marker.address = address;
                scope.marker.locationChanged = false;
              });
          }

          scope.edit = function () {
            showPrompt(scope);
          };

          return scope;
        },
        options: {
          noHide: true
        },
        isWorkLocation: aMarker.isWorkLocation
      };

      if (aMarker.isWorkLocation) {
        job.jsProperties.workLocationSet = true;
      }

      job.markerBounds.push([latitude, longitude]);

      return job.markersOnJob.length - 1;
    };


    return markers;
  });